@@ -2,6 +2,7 @@
 #  dependency injection ('scenarios') by tests.
 #
 # Copyright (c) 2009, Robert Collins <robertc@robertcollins.net>
+# Copyright (c) 2010, 2011 Martin Pool <mbp@sourcefrog.net>
 # 
 # Licensed under either the Apache License, Version 2.0 or the BSD 3-clause
 # license at the users choice. A copy of both licenses are available in the
@@ -21,6 +22,8 @@ from testscenarios.scenarios import (
     apply_scenario,
     apply_scenarios,
     generate_scenarios,
+    load_tests_apply_scenarios,
+    multiply_scenarios,
     )
 import testtools
 from testtools.tests.helpers import LoggingResult
@@ -171,3 +174,88 @@ class TestApplyScenarios(testtools.TestCase):
         tests = list(apply_scenarios(ReferenceTest.scenarios, test))
         self.assertEqual([('demo', {})], ReferenceTest.scenarios)
         self.assertEqual(ReferenceTest.scenarios, tests[0].scenarios)
+
+
+class TestLoadTests(testtools.TestCase):
+
+    class SampleTest(unittest.TestCase):
+        def test_nothing(self): 
+            pass
+        scenarios = [
+            ('a', {}),
+            ('b', {}),
+            ]
+
+    def test_load_tests_apply_scenarios(self):
+        suite = load_tests_apply_scenarios(
+            unittest.TestLoader(),
+            [self.SampleTest('test_nothing')],
+            None)
+        result_tests = list(testtools.iterate_tests(suite))
+        self.assertEquals(
+            2,
+            len(result_tests),
+            result_tests)
+
+    def test_load_tests_apply_scenarios_old_style(self):
+        """Call load_tests in the way used by bzr."""
+        suite = load_tests_apply_scenarios(
+            [self.SampleTest('test_nothing')],
+            self.__class__.__module__,
+            unittest.TestLoader(),
+            )
+        result_tests = list(testtools.iterate_tests(suite))
+        self.assertEquals(
+            2,
+            len(result_tests),
+            result_tests)
+
+
+class TestMultiplyScenarios(testtools.TestCase):
+
+    def test_multiply_scenarios(self):
+        def factory(name):
+            for i in 'ab':
+                yield i, {name: i}
+        scenarios = multiply_scenarios(factory('p'), factory('q'))
+        self.assertEqual([
+            ('a,a', dict(p='a', q='a')),
+            ('a,b', dict(p='a', q='b')),
+            ('b,a', dict(p='b', q='a')),
+            ('b,b', dict(p='b', q='b')),
+            ],
+            scenarios)
+
+    def test_multiply_many_scenarios(self):
+        def factory(name):
+            for i in 'abc':
+                yield i, {name: i}
+        scenarios = multiply_scenarios(factory('p'), factory('q'),
+            factory('r'), factory('t'))
+        self.assertEqual(
+            3**4,
+            len(scenarios),
+            scenarios)
+        self.assertEqual(
+            'a,a,a,a',
+            scenarios[0][0])
+
+
+class TestPerModuleScenarios(testtools.TestCase):
+
+    def test_per_module_scenarios(self):
+        """Generate scenarios for available modules"""
+        s = testscenarios.scenarios.per_module_scenarios(
+            'the_module', [
+                ('Python', 'testscenarios'),
+                ('unittest', 'unittest'),
+                ('nonexistent', 'nonexistent'),
+                ])
+        self.assertEqual('nonexistent', s[-1][0])
+        self.assertIsInstance(s[-1][1]['the_module'], tuple)
+        s[-1][1]['the_module'] = None
+        self.assertEqual(s, [
+            ('Python', {'the_module': testscenarios}),
+            ('unittest', {'the_module': unittest}),
+            ('nonexistent', {'the_module': None}),
+            ])
